package cz.muni.fi.pb162.test3;

/**
 * Toto rozhrani reprezentuje film nebo jine dilo, ktery muze byt predmetem 
 * hodnoceni podle MPAA film rating system. Tento system klasifikuje filmy na 
 * zaklade jejich vhodnosti pro deti a mladez.
 *
 * Viz <a href="http://en.wikipedia.org/wiki/MPAA_rating_system"
 * >http://en.wikipedia.org/wiki/MPAA_rating_system</a>.
 *
 * @author Petr Adamek
 */
public interface RatableMovie {
    
    /**
     * Tento vyctovy typ reprezentuje jednotlive stupne klasifikace dle
     * MPAA film rating system.
     */
    public enum Rating { 
        
        /**
         * Film je urcen pro kazdeho.
         */
        G, 
        
        /**
         * Nektere sceny nemusi byt vhodne pro deti, je doporuceny dohled rodicu.
         */
        PG, 
        
        /**
         * Nektere sceny jsou nevhodne pro deti do 13 let, rodice by meli byt 
         * velmi obezretni.
         */
        PG_13, 
        
        /**
         * Nevhodne pro osoby mladsi 17 let bez doprovodu rodicu nebo dospele osoby.
         */        
        R, 
        
        /**
         * Neprijatelne pro osoby mladsi 18 let.
         */
        NC_17 
    };
    
    /**
     * Vrati klasifikaci filmu dle MPAA film rating system.
     *
     * @return klasifikaci filmu dle MPAA film rating system
     */
    public Rating getRating();
    
}
    