package cz.muni.fi.pb162.test3;

import java.io.File;
import java.io.InputStream;
import java.io.OutputStream;

/**
 * Toto rozhrani reprezentuje tridu, ktera umi obsah katalogu 
 * v nejakem obchodu ulozit do souboru, nebo ho ze souboru opet nacist.
 * 
 * Souboru obsahuje jeden produkt na jednom radku. Nejdrive je pocet kusu na 
 * sklade, potom cena a zbytek tvori nazev produktu. Poradi radku nehraje roli. 
 * Soubor je ulozen v kodovani UTF-8. 
 *
 * Pro jednoduchost predpokladejte, ze katalog obsahuje pouze obycejne produkty, 
 * pripadne potomky tridy product ignorujte.
 *
 * @author Petr Adamek
 */
public interface ShopStorage {

    /**
     * Ulozi obsah katalogu do zadaneho binarniho proudu.
     *
     * @param shop obchod, jehoz katalog se ma ulozit.
     * @param outputStream binarni proud, do nejz se ma katalog ulozit
     * @throws cz.muni.fi.pb162.test3.ShopStorageException pokud dojde k chybe pri zapisu
     */
    public void saveShop(SimpleShop shop, OutputStream outputStream) throws 
            ShopStorageException;

    /**
     * Ulozi obsah katalogu do zadaneho souboru.
     *
     * @param shop obchod, jehoz katalog se ma ulozit.
     * @param file soubor, do nejz se ma katalog ulozit
     * @throws cz.muni.fi.pb162.test3.ShopStorageException pokud dojde k chybe pri zapisu
     */
    public void saveShop(SimpleShop shop, File file) throws 
            ShopStorageException;

    /**
     * Nacte obsah katalogu nejakeho obchodu z binarniho proudu.
     *
     * @return obchod, jehoz katalog byl nacten ze souboru
     * @param inputStream binarni proud, z nehoz se ma obsah katalogu nacist
     * @throws cz.muni.fi.pb162.test3.InvalidFormatException pokud ma vstupni soubor spatny format
     * @throws cz.muni.fi.pb162.test3.ShopStorageException pokud dojde k chybe pri cteni
     */

    public SimpleShop loadShop(InputStream inputStream) throws 
            ShopStorageException, InvalidFormatException;;

    /**
     * Nacte obsah katalogu nejakeho obchodu ze souboru.
     *
     * @return obchod, jehoz katalog byl nacten ze souboru
     * @param file soubor, z nehoz se ma obsah katalogu nacist
     * @throws cz.muni.fi.pb162.test3.ShopStorageException pokud dojde k chybe pri cteni
     * @throws cz.muni.fi.pb162.test3.InvalidFormatException pokud ma vstupni soubor spatny format
     */
    public SimpleShop loadShop(File file) throws 
            ShopStorageException, InvalidFormatException;;

}
