package cz.muni.fi.pb162.test3;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import junit.framework.TestCase;

/**
 *
 * @author Petr Adamek
 */
public class ShopStorageTest extends TestCase {
    
    public ShopStorageTest(String testName) {
        super(testName);
    }
    
    private static final String [] TEST_DATA = {
        "0 123.50 Liska Bystrouska (kniha)",
        "101 10 Pohrabac kovovy",
        "11 236000000 Lokomotiva 363",
        "0 11.50 Radegast lahev, 12 stupnu",
    };
    
    private static final String [] TEST_PRODUCTS_NAMES = {
        "Liska Bystrouska (kniha)",
        "Pohrabac kovovy",
        "Lokomotiva 363",
        "Radegast lahev, 12 stupnu",
    };
    
    private static final BigDecimal[] TEST_PRODUCTS_PRICES = {
        BigDecimal.valueOf(12350,2),
        BigDecimal.valueOf(10),
        BigDecimal.valueOf(236000000),
        BigDecimal.valueOf(1150,2),
    };
    
    private static final int[] TEST_PRODUCTS_COUNT = {
        0,101,11,0
    };
    
    private Product findProduct(Collection<Product> products, String name) {
        for (Product p : products) {
            if (p.getName().equals(name)) {
                return p;
            }
        }
        return null;
    }
    
    public void testLoadFromStream() throws ShopException, IOException {
        ShopStorage storage = new ShopStorageImpl();
        InputStream is = createTestInputStream(TEST_DATA);
        testLoadResult(storage.loadShop(is));
    }
    
    public void testLoadFromFile() throws ShopException, IOException {
        ShopStorage storage = new ShopStorageImpl();
        File file = createTestInputFile(TEST_DATA);
        testLoadResult(storage.loadShop(file));
    }
    
    public void testLoadResult(SimpleShop shop) throws ShopException {
        Set<Product> loadedProducts = shop.getAllProducts();
        for(int i = 0; i < TEST_PRODUCTS_NAMES.length; i++) {
            String name = TEST_PRODUCTS_NAMES[i];
            Product p = findProduct(loadedProducts,name);
            assertNotNull("Pri nacitani se ztratil product " + name + ", " +
                    "nacteny byly pouze tyto produkty " + loadedProducts,p);
            assertEquals(TEST_PRODUCTS_COUNT[i],shop.getProductItemsCount(p));
            assertEquals(TEST_PRODUCTS_PRICES[i],p.getPrice());
        }
    }
    
    public void testSaveToFile() throws ShopException, IOException {
        testSave(true);
    }
    
    public void testSaveToOutputStream() throws ShopException, IOException {
        testSave(false);
    }
    
    public void testSave(boolean file) throws ShopException, IOException {
        SimpleShop shop = new SimpleShopImpl();
        
        for(int i = 0; i < TEST_PRODUCTS_NAMES.length; i++) {
            Product p = new Product(TEST_PRODUCTS_NAMES[i],TEST_PRODUCTS_PRICES[i]);
            shop.addProduct(p,TEST_PRODUCTS_COUNT[i]);
        }
        
        ShopStorage storage = new ShopStorageImpl();
        ByteArrayInputStream data;
        
        if (file) {
            byte[] buffer = new byte[4096];
            File testFile = createTempFile();
            storage.saveShop(shop,testFile);
            InputStream tempStream = new FileInputStream(testFile);
            int size = tempStream.read(buffer);
            if (tempStream.read() != -1) {
                throw new RuntimeException("test selhal z duvodu male velikosti testovaciho bufferu!");
            }
            data = new ByteArrayInputStream(buffer,0,size);
        } else {
            ByteArrayOutputStream testStream = new ByteArrayOutputStream();
            storage.saveShop(shop,testStream);
            data = new ByteArrayInputStream(testStream.toByteArray());
        }
        
        Set<String> testDataLines = new HashSet<String>(Arrays.asList(TEST_DATA));
        
        BufferedReader reader = new BufferedReader(new InputStreamReader(data, ENCODING));
        
        String line;
        while ((line = reader.readLine()) != null) {
            assertTrue("Vystupni soubor obsahuje radek, ktery nebyl ve vstupnich" +
                    " testovacich datech: " + line,testDataLines.contains(line));
            testDataLines.remove(line);
        };
        assertTrue("Ve vystupnim souboru chybi nektere radky ze vstupnich " +
                "testovacich dat: " + testDataLines,
                testDataLines.isEmpty());
        
    }
    
    public final void testLoadWrongFormat() throws IOException, ShopStorageException {
        try {
            new ShopStorageImpl().loadShop(createTestInputStream(""));
        } catch (InvalidFormatException ex) {
            // All is ok
        }
        try {
            new ShopStorageImpl().loadShop(createTestInputStream("1"));
        } catch (InvalidFormatException ex) {
            // All is ok
        }
        try {
            new ShopStorageImpl().loadShop(createTestInputStream("1 10.50"));
        } catch (InvalidFormatException ex) {
            // All is ok
        }
        try {
            new ShopStorageImpl().loadShop(createTestInputStream("a 100.4 cdscs"));
        } catch (InvalidFormatException ex) {
            // All is ok
        }
        try {
            new ShopStorageImpl().loadShop(createTestInputStream("1 aa cdscs"));
        } catch (InvalidFormatException ex) {
            // All is ok
        }
    }
    
    public final void testLoadIOError() {
        try {
            new ShopStorageImpl().loadShop(new BadInputStream());
            fail("Vyjimka IOException pri cteni ze vstupniho proudu" +
                    " je zrejme ignorovana!!! " +
                    "(nezpusobi vyvolani pozadovane vyjimky ShopStorageException)");
        } catch (ShopStorageException ex) {
            assertSame("Vyjimka IOException pri cteni ze vstupniho proudu " +
                    "sice zpusobi vyvolani vyjimky ShopStorageException, ale tato " +
                    "vyjimka nema spravne nastavenou svoji pricinu (viz retezeni vyjimek)",
                    IOEXCEPTION,ex.getCause());
        } catch (Exception ex) {
            fail("Vyjimka IOException pri cteni ze vstupniho proudu" +
                    " zpusobi vyvolani vyjimky " + ex.getClass() + " misto pozadovane " +
                    "ShopStorageException");
        }
    }
    
    public final void testSaveIOError() {
        SimpleShop shop = new SimpleShopImpl();
        shop.addProduct(new Product("neco",BigDecimal.valueOf(10)),1);
        try {
            new ShopStorageImpl().saveShop(shop,new BadOutputStream());
            fail("Vyjimka IOException pri zapisu do vystupniho proudu" +
                    " je zrejme ignorovana!!! " +
                    "(nezpusobi vyvolani pozadovane vyjimky ShopStorageException)." +
                    " Nepouzivate nahodou tridu PrintWriter, ktera IOException " +
                    "negeneruje? ");
        } catch (ShopStorageException ex) {
            assertSame("Vyjimka IOException pri zapisu do vystupniho proudu " +
                    "sice zpusobi vyvolani vyjimky ShopStorageException, ale tato " +
                    "vyjimka nema spravne nastavenou svoji pricinu (viz retezeni vyjimek)",
                    IOEXCEPTION,ex.getCause());
        } catch (Exception ex) {
            fail("Vyjimka IOException pri zapisu dp vystupniho proudu" +
                    " zpusobi vyvolani vyjimky " + ex.getClass() + " misto pozadovane " +
                    "ShopStorageException");
        }
    }
    
    public static final String ENCODING = "UTF-8";
    
    protected static byte[] stringLinesToByteArray(String ... lines) throws IOException {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        Writer writer = new OutputStreamWriter(out,ENCODING);
        for (String line : lines) {
            writer.write(line);
            writer.write("\n");
        }
        writer.flush();
        return out.toByteArray();
    }
    
    private static InputStream createTestInputStream(String ... lines) throws
            IOException {
        return new ByteArrayInputStream(stringLinesToByteArray(lines));
    }
    
    private static File createTempFile() throws IOException {
        File result = File.createTempFile("pb162","txt");
        result.deleteOnExit();
        return result;
    }
    
    private static File createTestInputFile(String ... lines) throws
            IOException {
        File f = createTempFile();
        FileOutputStream out = new FileOutputStream(f);
        out.write(stringLinesToByteArray(lines));
        out.flush();
        out.close();
        return f;
    }
    
    private static final IOException IOEXCEPTION =
            new IOException("It is not possible to read from this stream");
    
    private class BadInputStream extends InputStream {
        
        public int read() throws IOException {
            throw IOEXCEPTION;
        }
    }
    
    private class BadOutputStream extends OutputStream {
        
        public void write(int value) throws IOException {
            throw IOEXCEPTION;
        }
    }
    
}
