package cz.muni.fi.pb162.test3;

import java.util.Set;

/**
 * Toto rozhrani reprezentuje jednoduchy obchod. Jeho zakladem je katalog 
 * produktu. U kazdeho produktu se take eviduje pocet volnych kusu na sklade, 
 * tento pocet muze byt i nulovy, coz znamena, ze je produkt v nabidce, 
 * ale neni na sklade (pripadne je jiz objednan jinym zakaznikem). 
 *
 * @author Petr Adamek
 */
public interface SimpleShop {
    
    /**
     * Pridany zadany pocet kusu daneho produktu do katalogu. Pocet kusu muze 
     * byt i nula, pak se produkt jenom prida do nabidky, aniz by byl pridan 
     * do skladu.
     *
     * @param product prodkut, ktery se ma pridat do katalogu
     * @param count pocet kusu
     * @throws IllegalArgumentException pokud je count mensi nez nula.
     * @throws NullPointerException pokud je product null
     */
    void addProduct(Product product, int count);

    /**
     * Vrati pocet kusu daneho produktu na sklade. Pokud produkt neni na sklade, 
     * vrati 0. Pokud neni ani v nabidce, vyhodi vyjimku.
     *
     * @return pocet kusu daneho produktu na sklade
     * @param product produkt, jehoz pocet kusu skladem chci znat
     * @throws cz.muni.fi.pb162.test3.UnknownProductException pokud neni dany 
     * produkt vubec v nabidce
     * @throws NullPointerException pokud je product null
     */
    int getProductItemsCount(Product product) throws UnknownProductException;
    
    /**
     * Vrati vsechny produkty v nabidce.
     *
     * @return vsechny produkty v nabidce
     */
    Set<Product> getAllProducts();
    
    /**
     * Vrati vsechny produkty, ktere jsou skladem.
     *
     * @return vsechny produkty, ktere jsou skladem
     */
    Set<Product> getAvailableProducts();

    /**
     * Vrati vsechny produkty, ktere jsou skladem a ktere lze prodat detem.
     * Detem lze prodat cokoliv krome filmu s ratingem jinym nez G.
     *
     * @return vsechny produkty, ktere jsou skladem a ktere lze prodat detem
     */
    Set<Product> getAvailableProductsForChildren();

    /**
     * Metoda umoznuje objednat dane produkty. Pokud je mezi produkty nektery 
     * produkt vicekrat, objedna se prislusny pocet kusu. Objednani probehne 
     * vytvorenim objednavky (trida Order). Pri objednani se zaroven odebere 
     * prislusny pocet kusu kazdeho produktu ze skladu. Objednat je mozne 
     * i produkty, ktere nejsou skladem.
     * 
     * Pokud byl objednan posledni produkt v katalogu, produkt zustava stale v
     * nabidce, pouze se pocet kusu na sklade snizi na nula. Pozor, metoda
     * musi selhat atomicky, tj. pokud se objednavku nepodari dokoncit, nesmi
     * se nic zrusit ze skladu!
     *
     * @return vytvorenou objednavku
     * @param products produkty, ktere chci objednat.
     * @throws cz.muni.fi.pb162.test3.UnknownProductException pokud je v 
     * objednavce neznamy produkt
     * @throws NullPointerException pokud je nektery produkt null
     */
    public Order order(Product ... products) throws UnknownProductException;

}
