package cz.muni.fi.pb162.test3;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;
import junit.framework.TestCase;

/**
 * @author Petr Adamek
 */
public class SimpleShopTest extends TestCase {
    
    public SimpleShopTest(String testName) {
        super(testName);
    }
    
    Product productA = new Product("A",BigDecimal.valueOf(1000,2));
    Product productB = new Product("B",BigDecimal.valueOf(1250,2));
    Product moviePohadka = new Movie("Krtecek",BigDecimal.valueOf(69950,2),Movie.Rating.G);
    Product moviePohadka2 = new Movie("Pysna princezna",BigDecimal.valueOf(69950,2),Movie.Rating.G);
    Product moviePorno = new Movie("Tatiana 2",BigDecimal.valueOf(59950,2),Movie.Rating.NC_17);
    
//    public void assertEquals(Set<Product> expected, Set<Product> actual) {
//        assertEquals((Object) expected,(Object) actual);
//    }
    
    public void testAddProduct() throws UnknownProductException {
        SimpleShopImpl shop = new SimpleShopImpl();
        shop.addProduct(productA,0);
        assertEquals(0,shop.getProductItemsCount(productA));
        shop.addProduct(productA,0);
        assertEquals(0,shop.getProductItemsCount(productA));
        shop.addProduct(productA,5);
        assertEquals(5,shop.getProductItemsCount(productA));
        shop.addProduct(productA,5);
        assertEquals(10,shop.getProductItemsCount(productA));
        shop.addProduct(productB,0);
        assertEquals(10,shop.getProductItemsCount(productA));
        assertEquals(0,shop.getProductItemsCount(productB));
        shop.addProduct(productB,5);
        assertEquals(10,shop.getProductItemsCount(productA));
        assertEquals(5,shop.getProductItemsCount(productB));
        shop.addProduct(moviePohadka,11);
        assertEquals(10,shop.getProductItemsCount(productA));
        assertEquals(5,shop.getProductItemsCount(productB));
        assertEquals(11,shop.getProductItemsCount(moviePohadka));
    }

    public void testAddProductExceptions() {
        SimpleShopImpl shop = new SimpleShopImpl();
        try {
            shop.addProduct(productA,-1);
            fail("Ocekava se IllegalArgumentException");
        } catch(IllegalArgumentException ex) {
        }
        
        try {
            shop.addProduct(productA,-1271);
            fail("Ocekava se IllegalArgumentException");
        } catch(IllegalArgumentException ex) {
        }

        try {
            shop.addProduct(null,1);
            fail("Ocekava se NullPointerException");
        } catch(NullPointerException ex) {
        }
    }
    
    public void testGetProductItemsCount() throws UnknownProductException {
        SimpleShopImpl shop = new SimpleShopImpl();
        
        try {
            shop.getProductItemsCount(null);
            fail("Ocekava se NullPointerException");
        } catch(NullPointerException ex) {
        }
        try {
            shop.getProductItemsCount(productA);
            fail("Ocekava se UnknownProductException");
        } catch(UnknownProductException ex) {
        }
        try {
            shop.getProductItemsCount(productB);
            fail("Ocekava se UnknownProductException");
        } catch(UnknownProductException ex) {
        }
        shop.addProduct(productA,0);
        assertEquals(0,shop.getProductItemsCount(productA));
        try {
            shop.getProductItemsCount(productB);
            fail("Ocekava se UnknownProductException");
        } catch(UnknownProductException ex) {
        }
    }
    
    private <T> Set<T> createSet(T ... values) {
        return new HashSet<T>(Arrays.asList(values));
    }
    
    public void testGetAllProducts()  {
        SimpleShopImpl shop = new SimpleShopImpl();
        assertEquals(Collections.emptySet(),shop.getAllProducts());
        shop.addProduct(productA,0);
        assertEquals(createSet(productA),shop.getAllProducts());
        shop.addProduct(productA,10);
        assertEquals(createSet(productA),shop.getAllProducts());
        shop.addProduct(productB,11);
        assertEquals(createSet(productA,productB),shop.getAllProducts());
        shop.addProduct(moviePohadka,0);
        assertEquals(createSet(productA,productB,moviePohadka),shop.getAllProducts());
        shop.addProduct(moviePorno,8);
        assertEquals(createSet(productA,productB,moviePohadka,moviePorno),shop.getAllProducts());
    }

    public void testGetAllProductsReadOnly()  {
        try {
            new SimpleShopImpl().getAllProducts().clear();
            fail("getAllProducts nevraci readonly kolekci");
        } catch(UnsupportedOperationException ex){
        }
    }

    public void testGetAvailableProducts()  {
        SimpleShopImpl shop = new SimpleShopImpl();
        assertEquals(Collections.emptySet(),shop.getAvailableProducts());
        shop.addProduct(productA,0);
        assertEquals(Collections.emptySet(),shop.getAvailableProducts());
        shop.addProduct(productA,10);
        assertEquals(createSet(productA),shop.getAvailableProducts());
        shop.addProduct(productB,11);
        assertEquals(createSet(productA,productB),shop.getAvailableProducts());
        shop.addProduct(moviePohadka,0);
        assertEquals(createSet(productA,productB),shop.getAvailableProducts());
        shop.addProduct(moviePorno,8);
        assertEquals(createSet(productA,productB,moviePorno),shop.getAvailableProducts());
    }
    
    public void testGetAvailableProductsForChildren()  {
        SimpleShopImpl shop = new SimpleShopImpl();
        Product testProductForChildren = new TestRatableProduct(RatableMovie.Rating.G);
        Product testProductForAdult = new TestRatableProduct(RatableMovie.Rating.NC_17);
        
        assertEquals(Collections.emptySet(),shop.getAvailableProductsForChildren());
        shop.addProduct(productA,0);
        assertEquals(Collections.emptySet(),shop.getAvailableProductsForChildren());
        shop.addProduct(productA,10);
        assertEquals(createSet(productA),shop.getAvailableProductsForChildren());
        shop.addProduct(productB,11);
        assertEquals(createSet(productA,productB),shop.getAvailableProductsForChildren());
        shop.addProduct(moviePohadka,0);
        assertEquals(createSet(productA,productB),shop.getAvailableProductsForChildren());
        shop.addProduct(moviePorno,8);
        assertEquals(createSet(productA,productB),shop.getAvailableProductsForChildren());
        shop.addProduct(moviePohadka2,8);
        assertEquals(createSet(productA,productB,moviePohadka2),shop.getAvailableProductsForChildren());
        shop.addProduct(testProductForAdult,1);
        assertEquals(createSet(productA,productB,moviePohadka2),shop.getAvailableProductsForChildren());
        shop.addProduct(testProductForChildren,1);
        assertEquals(createSet(productA,productB,moviePohadka2,testProductForChildren),shop.getAvailableProductsForChildren());
    }

    private Order createOrder(Product ... product) {
        Order order = new Order();
        for(Product p : product) {
            order.addItem(p,1);
        }
        return order;
    }
    
    public void testOrder() throws UnknownProductException {
        SimpleShopImpl shop = new SimpleShopImpl();

        shop.addProduct(productA,0);
        shop.addProduct(productB,27);
        shop.addProduct(moviePorno,12);
        shop.addProduct(moviePohadka2,2);
        shop.addProduct(moviePohadka,1);
        
        assertEquals(
                createOrder(productA,productB,productA,moviePohadka2,moviePohadka2,moviePohadka2, moviePohadka),
                shop.order(productA,productB,productA,moviePohadka2,moviePohadka2,moviePohadka2, moviePohadka));

        assertEquals(createSet(productA,productB,moviePohadka,moviePohadka2,moviePorno),shop.getAllProducts());
        assertEquals(createSet(productB,moviePorno),shop.getAvailableProducts());
        assertEquals(createSet(productB),shop.getAvailableProductsForChildren());

        assertEquals(0,shop.getProductItemsCount(productA));
        assertEquals(26,shop.getProductItemsCount(productB));
        assertEquals(12,shop.getProductItemsCount(moviePorno));
        assertEquals(0,shop.getProductItemsCount(moviePohadka2));
        assertEquals(0,shop.getProductItemsCount(moviePohadka));

        assertEquals(
                createOrder(moviePorno),
                shop.order(moviePorno));
        
        assertEquals(createSet(productA,productB,moviePohadka,moviePohadka2,moviePorno),shop.getAllProducts());
        assertEquals(createSet(productB,moviePorno),shop.getAvailableProducts());
        assertEquals(createSet(productB),shop.getAvailableProductsForChildren());

        assertEquals(0,shop.getProductItemsCount(productA));
        assertEquals(26,shop.getProductItemsCount(productB));
        assertEquals(11,shop.getProductItemsCount(moviePorno));
        assertEquals(0,shop.getProductItemsCount(moviePohadka2));
        assertEquals(0,shop.getProductItemsCount(moviePohadka));
    }

    public void testOrderFailure() throws UnknownProductException {
        SimpleShopImpl shop = new SimpleShopImpl();

        shop.addProduct(productA,0);
        shop.addProduct(productB,27);
        shop.addProduct(moviePorno,12);
        shop.addProduct(moviePohadka2,2);
        shop.addProduct(moviePohadka,1);

        try {
            shop.order((Product) null);
            fail("Ocekava se NullPointerException");
        } catch(NullPointerException ex) {
        }
        try {
            shop.order(productB,null,moviePohadka);
            fail("Ocekava se NullPointerException");
        } catch(NullPointerException ex) {
        }
        try {
            shop.order(new Product("neznamy produkt",BigDecimal.valueOf(10)));
            fail("Ocekava se UnknownProductException");
        } catch(UnknownProductException ex) {
        }
    }

    public void testOrderFailAtomic() throws UnknownProductException {

        SimpleShopImpl shop = new SimpleShopImpl();

        shop.addProduct(productA,0);
        shop.addProduct(productB,27);
        shop.addProduct(moviePorno,12);
        shop.addProduct(moviePohadka2,2);
        shop.addProduct(moviePohadka,1);

        try {
            shop.order(productB,null,moviePohadka);
            fail("Ocekava se NullPointerException");
        } catch(NullPointerException ex) {
        }

        assertEquals(0,shop.getProductItemsCount(productA));
        assertEquals(27,shop.getProductItemsCount(productB));
        assertEquals(12,shop.getProductItemsCount(moviePorno));
        assertEquals(2,shop.getProductItemsCount(moviePohadka2));
        assertEquals(1,shop.getProductItemsCount(moviePohadka));

        try {
            shop.order(productB,moviePohadka,new Product("Neco",BigDecimal.valueOf(0)));
            fail("Ocekava se NullPointerException");
        } catch(UnknownProductException ex) {
        }

        assertEquals(0,shop.getProductItemsCount(productA));
        assertEquals(27,shop.getProductItemsCount(productB));
        assertEquals(12,shop.getProductItemsCount(moviePorno));
        assertEquals(2,shop.getProductItemsCount(moviePohadka2));
        assertEquals(1,shop.getProductItemsCount(moviePohadka));
        
    }
    
    private class TestRatableProduct extends Product implements RatableMovie {
        
        private RatableMovie.Rating rating;
        
        public TestRatableProduct(RatableMovie.Rating rating) {
            super("Produkt s ratingem " + rating, BigDecimal.valueOf(100));
            this.rating = rating;
        }
        
        public RatableMovie.Rating getRating() {
            return rating;
        }
        
    }
    
    
}

